-- Localization
local L = LibStub("AceLocale-3.0"):GetLocale("TreasuresOfDraenor")

-- Libraries
local config = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")

local addon, TOD = ...
local abs, format, type, tostring, next, select, pairs, ipairs, string, table, tinsert = math.abs, format, type, tostring, next, select, pairs, ipairs, string, table, tinsert

TOD.OptionsName = L["Treasures Of Draenor"]

local general = {
	order = 1,
	type = "group",
	name = TOD.OptionsName.." options :",
	args = {
		separator0 = {
		order = 1,
		name = "\n",
		type = "description",
		},
		DraenorTreasure = {
		order = 2,
		width = "full",
		name = L["ON/OFF "]..TOD.OptionsName,
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.enabled = val end,
        get = function(info) return TreasuresOfDraenorDB.enabled end,
		},
		separator1 = {
		order = 3,
		name = "\n",
		type = "description",
		},
		header1 = {
		order = 4,
		type = "header",
		name = L["Zones"],
		},
		separator2 = {
		order = 5,
		name = "\n",
		type = "description",
		},
		desc2 = {
		order = 6,
		type = "description",
		name = L["You can select the areas where you want to display the treasures."],
		},
		separator3 = {
		order = 7,
		name = "\n",
		type = "description",
		},
		mapIDhidden941 = {
		order = 8,
		name = L["Frostfire"],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.mapIDhidden[941] = val end,
        get = function(info) return TreasuresOfDraenorDB.mapIDhidden[941] end,
		},
		mapIDhidden947 = {
		order = 9,
		name = L["Shadowmoon Valley"],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.mapIDhidden[947] = val end,
        get = function(info) return TreasuresOfDraenorDB.mapIDhidden[947] end,
		},
		mapIDhidden949 = {
		order = 10,
		name = L["Gorgrond"],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.mapIDhidden[949] = val end,
        get = function(info) return TreasuresOfDraenorDB.mapIDhidden[949] end,
		},
		mapIDhidden946 = {
		order = 11,
		name = L["Talador"],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.mapIDhidden[946] = val end,
        get = function(info) return TreasuresOfDraenorDB.mapIDhidden[946] end,
		},
		mapIDhidden948 = {
		order = 12,
		name = L["Spires of Arak"],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.mapIDhidden[948] = val end,
        get = function(info) return TreasuresOfDraenorDB.mapIDhidden[948] end,
		},
		mapIDhidden950 = {
		order = 13,
		name = L["Nagrand"],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.mapIDhidden[950] = val end,
        get = function(info) return TreasuresOfDraenorDB.mapIDhidden[950] end,
		},
		mapIDhidden945 = {
		order = 14,
		name = L["Tanaan Jungle"],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.mapIDhidden[945] = val end,
        get = function(info) return TreasuresOfDraenorDB.mapIDhidden[945] end,
		},
		separator4 = {
		order = 15,
		name = "\n",
		type = "description",
		},
		header2 = {
		order = 16,
		type = "header",
		name = L["Minimap"],
		},
		separator5 = {
		order = 17,
		name = "\n",
		type = "description",
		},
		minimap = {
		order = 18,
		width = "full",
		name = L["ON/OFF display treasures on the minimap."],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.minimap = val end,
		get = function(info)	if TreasuresOfDraenorDB.minimap then
									MinimapPOIHandler:Enable()
								else
									MinimapPOIHandler:Disable()
								end
								return TreasuresOfDraenorDB.minimap end,
		},
		minimapAuto = {
		order = 19,
		width = "full",
		name = L["Reactive automatically displaying treasures."],
		desc = L["Reactive automatically display on the minimap treasure rebooting after each game if it was off."],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.minimapAuto = val end,
        get = function(info) return TreasuresOfDraenorDB.minimapAuto end,
		},
	},
}

local currentSelected, guiTreasures
local treasuresListe = {
	order = 2,
	type = "group",
	name = L["Treasures hidden options :"],
	args = {
		separator1 = {
		order = 1,
		name = "\n",
		type = "description",
		},
		addButton = {
			name = L["Add to Treasures hidden"],
			desc = L["Adds the nearest treasure our position has the hidden list."],
			order = 5,
			width = "full",
			type = "execute",
			func = function ()
				local closest, dist = GetClosestTreasure()
				if closest then
					TreasuresOfDraenorDB.hidden[closest.id] = true
					DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfDraenor : Treasure "]..closest.name.." ("..closest.id..") "..L["is now hidden"])
				else
					DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfDraenor : there's no treasure nearby"])
				end
			end,
		},
		separator2 = {
		order = 10,
		name = "\n",
		type = "description",
		},
		trashlist = {
			name = L["Click a treasure below to remove it from the list hidden."],
			order = 15,
			type = "group",
			inline = true,
			args = {
            theList = {
				type = "multiselect",
				name = L["Treasures hidden"],
				width = "full",
				values = function () return TOD:GuiMakeTreasures() end,
				get = function () return currentSelected end,          
				set = function (info, key) TOD:GuiRemoveFromTreasures(key) end,
				},
			},
		},
		separator3 = {
		order = 20,
		name = "\n",
		type = "description",
		},
		clearTreasures = {
			name = L["Clear ALL Treasures Hidden"],
            order = 25,
			width = "full",
            type = "execute",
            func = function() 
				table.wipe(TreasuresOfDraenorDB.hidden)
				DEFAULT_CHAT_FRAME:AddMessage(L["TreasuresOfDraenor : hidden treasures cleared"]) 
			end,
        },
	},                                                            
}

local tomtom = {
	order = 3,
	type = "group",
	name = "Tomtom options :",
	args = {
		desc1 = {
		order = 1,
		type = "description",
		name = L["This setting changes the modifier used by TomTom when left-clicking on the treasure map to create a waypoint"],
		},
		separator1 = {
		order = 2,
		name = "\n",
		type = "description",
		},
		modifier = {
		type = "select",
		order = 3,
		name = L["Select buttom"],
		desc = L["This setting changes the modifier used by TomTom when left-clicking on the treasure map to create a waypoint"],
		set	= function(info, val) TreasuresOfDraenorDB.tomtomKey = val end,
		get	= function(info) return TreasuresOfDraenorDB.tomtomKey end,
		values	= function()
		return {
			["A"] = "Alt + left-click",
			["C"] = "Ctrl + left-click",
			["S"] = "Shift + left-click",
		}
		end,
		},
	},
}

local debogage = {
	order = 4,
	type = "group",
	name = "Débogage options :",
	args = {
		separator4 = {
		order = 1,
		name = "\n",
		type = "description",
		},
		header3 = {
		order = 2,
		type = "header",
		name = "Débogage",
		},
		Debug = {
		order = 3,
		width = "full",
		name = L["ON/OFF débogage."],
		type = "toggle",
		set = function(info, val) TreasuresOfDraenorDB.debug = val end,
        get = function(info) return TreasuresOfDraenorDB.debug end,
		},
	},
}

function TOD:GuiMakeTreasures()
   local dbTreasures = TreasuresOfDraenorDB.hidden
   if ((not dbTreasures) or (not next(dbTreasures))) then
      Debug("The DB Trashlist is empty.  Returning {}.")
      guiTreasures = {}
      return guiTreasures
   end
   guiTreasures = {}
   for i, v in pairs(TOD.treasures) do
		if dbTreasures[v.id] then
			guiTreasures[i] = L["Treasure ID : "]..v.id.." - "..v.name
		end
   end
   return guiTreasures
end

function TOD:GuiRemoveFromTreasures(key)
	local treasureHidden = guiTreasures[key]
	if treasureHidden then
		for i, v in pairs(TOD.treasures) do
			local dbTreasures = TreasuresOfDraenorDB.hidden
			local name = treasureHidden
			if (name and dbTreasures and name == L["Treasure ID : "]..v.id.." - "..v.name) then
				TreasuresOfDraenorDB.hidden[v.id] = nil
				DEFAULT_CHAT_FRAME:AddMessage(L["Treasure ID : "]..v.id..L[" was deleted from the list."])
			end
		end
	end
end

local function findPanel(name, parent)
	for i, button in next, InterfaceOptionsFrameAddOns.buttons do
		if button.element then
			if name and button.element.name == name then return button
			elseif parent and button.element.parent == parent then return button
			end
		end
	end
end

function TOD:LoadOptions()
	if (config and dialog) then 
		-- Add options to Bliz addon config frame
		-- General Options
		config:RegisterOptionsTable(TOD.OptionsName, general)
		local options = dialog:AddToBlizOptions(TOD.OptionsName, TOD.OptionsName)
		options:HookScript("OnShow", function()
		local p = findPanel(TOD.OptionsName)
		if p and p.element.collapsed then OptionsListButtonToggle_OnClick(p.toggle) end
		end)
		
		-- Treasures List Options
		config:RegisterOptionsTable("TOD/treasuresListe", treasuresListe)
		dialog:AddToBlizOptions("TOD/treasuresListe", L["Treasures hidden"], TOD.OptionsName)
		
		-- TOMTOM Options
		if _G.TomTom then
			config:RegisterOptionsTable("TOD/tomtom", tomtom)
			dialog:AddToBlizOptions("TOD/tomtom", "Tomtom", TOD.OptionsName)
		end
		
		-- Debogage Options
		config:RegisterOptionsTable("TOD/debogage", debogage)
		dialog:AddToBlizOptions("TOD/debogage", "Débogage", TOD.OptionsName)
   end
end